
//////////////////////////////////////////////////////////////////
// Init
//////////////////////////////////////////////////////////////////
void vic_init()
{
        for(int x=0;x<320;x++)
        {
                for(int y=0;y<200;y++)
                {
                        screen[x][y]=0;
                }
        }
}
//////////////////////////////////////////////////////////////////
// TextMode functions
//////////////////////////////////////////////////////////////////

void vic_drawTextModeScreen()
{
		
		
		//
		//get the char codes from the screen memory
		//
		
		unsigned short memory_pos = VicBank + VicScreenMemory;

		//character number + charcter memory start address + bank address
		//holds the foreground color of the chars 
		CharColorByte	= 0;
	
		//draw the chars
        for( int y=0; y<200; y+=8 )
        {
                for( int x=0; x<320; x+=8 )
                {
                        byte charCode = peek( memory_pos++ );
						if(!MultiColorMode && !UserMcmMode)
							vic_paintChar( charCode ,x,y );
                        if(MultiColorMode || UserMcmMode)
							vic_paintMCMChar( charCode ,x,y );						
                }
        }
}
void vic_paintChar( byte charNr, int xPos, int yPos)
{
	/*
		- every byte in the pointmatrix is splitted in 8 bits 
		  with 1 of 2 seperate color infos (0 or 1)
		- 0=background
		- 2=foreground
		- so 8 bytes form an 8x8 pixel character
	*/
	//first a little array for reversing bits later
	byte bits[]		= {7,6,5,4,3,2,1,0};
	//now we get the 2 static colors for this char
	byte bg_color	= peek(0xD021);
	byte fg_color	= peek(0xD800+(CharColorByte++)) & 0xF;

	//lets start the draw routine
	for( int y=0; y<8; y++) // the char height is 8 pixel
	{
		//now we get the byte for the current line from the char set memory
		byte CharPointByte	= peek(VicBank + VicAddress + (charNr * 8) + y);
		for( int x=0; x<8; x++) // the char width is 8 pixel
		{
			//here we check the bit of the "line-byte" ( 8 bits of course ) for its value
			//and setting the color (we have to reverse the bits here)
			byte ColorValue	= TestBit(CharPointByte, bits[x]);
			//now that we have the color bit lets look what color it is
			if(ColorValue == 0) //Bit = 0 means that we select a Background Color
                ColorValue	= bg_color; 
			else if(ColorValue == 1) //Bit = 1 means we have to select a foreground color
				ColorValue	= fg_color; //&F because only the 
										//low four bits are used 
										//high 4 bits are drown away
			//at last we set the fetched color in our Screen Matrix
            screen[xPos + x][yPos + y] = ColorValue;
		}
	}
}
int vic_paintMCMChar( byte charNr, int xPos, int yPos)
{
	/*
		- every byte in the pointmatrix is splitted in 4 pairs of 2 bits 
		  with 4 possible color infos (00, 01, 10, 11)
		- 00: Background color 0 ($d021)
		- 01: Background color 1 ($d022)
		- 10: Background color 2 ($d023)
		- 11: Color Data (0xD800)
		- every pixel is streched to 2 pixels with the same color infos
	*/
	//first a little array for reversing bits later
	byte bits[]	= {7, 6, 5, 4, 3, 2, 1, 0};
	//if color nybble is less 8 the char is displayed normaly
	byte cdata		= peek(0xD800+CharColorByte) & 0xF;
	if(cdata < 8)
	{
        vic_paintChar( charNr ,xPos,yPos );
		return 0;
	}
	//lets start the draw routine
	for( int y=0; y<8; y++) // the char height is 8 pixel
	{
		//now we get the byte for the current line
		byte CharPointByte	= peek(VicBank + VicAddress + (charNr * 8 + y));
		for( int x=0; x<8; x+=2) // the char width is 4 pixel stretched to 8
		{
			//here we check the bit of the "line-byte" ( 8 bits of course ) for its value
			//and setting the color (we have to reverse the bits here)
			byte ColorValue1	= TestBit(CharPointByte, bits[x]);
			byte ColorValue2	= TestBit(CharPointByte, bits[x+1]);
			byte ColorValue		= (ColorValue1 << 1) | ColorValue2;
			if(ColorValue == 0)		 //00: Background color 0 ($d021)
                ColorValue	= peek(0xD021); 
			else if(ColorValue == 1) //01: Background color 1 ($d022)
				ColorValue	= peek(0xD022); 
			else if(ColorValue == 2) //10: Background color 2 ($d023)
                ColorValue	= peek(0xD023); 
			else if(ColorValue == 3) //11: Color Data (0xD800)
                ColorValue	= peek(0xD800+CharColorByte) & 0xF; 
			//at last we set the fetched color in our Screen Matrix
            screen[xPos + x][yPos + y]		= ColorValue;
			screen[xPos + x + 1][yPos + y]	= ColorValue;
		}
	}
	CharColorByte++;
	return 0;
}
//////////////////////////////////////////////////////////////////
// Sprite functions
//////////////////////////////////////////////////////////////////
void vic_drawSpriteScreen()
{
	/*
		- we have 8 sprites, each 24 x 21 pixel
		- the dot data for each sprite is an 64 byte area (only 63 used), 3 bytes form a sprite line
		- the dot area is called sprite datablock
		- to address the data block we need a pointer for each sprite to the data block address
		- sprite data pointer are located in the last 8 bytes of the screen memory relative to the vic bank
		- so the sprite data block start address will be VicBank + pointer
		- the bits of 0xD015 say if a sprite will be drawn or not
		- the bits of 0xD017 say if a sprite is stretched horizontal x 2 or not
		- the bits of 0xD01D say if a sprite is stretched vertical x 2 or not
		- the bits of 0xD01C say if a sprite uses multicolor or not
		- the bits of 0xD01B say if what a priority a sprite has
		- in normal color mode the sprite colors are the 8 bytes from 0xD027 to 0xD02E
		- the X and Y coordinates can taken from the registers 0xD000 to 0xD00F
		- because the screen resolution 320 is higher the a byte, we add a bit from
		  0xD010 to to the X coordinate byte..so we have 9 bits forming the X coord.
		- the sprite 0,0 point is in the upper right corner
	*/
	//we start our loop to fetch the data for the 8 sprites
	for(int s=0; s<8; s++)
	{
		//get the sprite pointer
		short SpritePointer		= peek(VicBank + VicScreenMemory + 0x400 - 8 + s);
		//lets get the start address for the dot area
		short DotAddress		= VicBank + SpritePointer * 64; 
		//now we check if the sprite will be drawn or not
		bool DrawSprite			= TestBit(peek(0xD015), s);
		//we check if this sprite will be stretched horizontal
		bool StretchHor			= TestBit(peek(0xD01D), s);
		//the same for vertical stretching
		bool StretchVer			= TestBit(peek(0xD017), s);
		//uses the sprite the multicolor mode?
		bool MultiColor			= TestBit(peek(0xD01C), s);
		//the sprite priority please...
		bool SpritePrior		= TestBit(peek(0xD01B), s);
		//lets fetch the X-coordinates for the sprite (remember the 9. bit)
		short XCoord			= peek(0xD000 + s*2) + (TestBit(peek(0xD010), s)?256:0);
			  XCoord			-= 24;
		//we fetch the Y-coordinates for this sprite
		short YCoord			= peek(0xD000 + s*2 + 1);
			  YCoord			-= 50;
		//now we draw our sprite...first only if it will be in normal color mode
		if(!MultiColor && DrawSprite && !UserMcmMode)
		{
			//we compute the needed hor. and vert. lines depending on stretching
			byte rows			= StretchVer?42:21;
			byte cols			= StretchHor?48:24;
			//we get the foreground color
			byte fgColor		= peek(0xD027 + s);
			//this will be the bit to test
			byte BitToTest		= 0;
			//a little array for reversing bits later
			byte bits[]		= {7,6,5,4,3,2,1,0};
			//now we loop through the sprite dot area
			for(int h=0; h<rows; h+=(rows/21))
			{
				for(int w=0; w<cols; w+=(cols/24))
				{
					//MemoryValue.Format("%X", DotAddress);
					//if(infoDlg)debug("DotAddress for this sprite: 0x"+MemoryValue);
					//we test if the dot is setted (bit=1) or not
					//only if it is 1 the color will be set, 0 = transparent = no color
					if(TestBit(peek(DotAddress), bits[BitToTest++]))
						if( (XCoord) >= 0		&&
							(XCoord+w+1) <= 320	&&
							(YCoord) >= 0		&&
							(YCoord+h+1) <= 200)
						{
							screen[XCoord+w][YCoord+h]			= fgColor;
							if(StretchHor)
								screen[XCoord+w+1][YCoord+h]	= fgColor;
							if(StretchVer)
								screen[XCoord+w][YCoord+h+1]	= fgColor;
							if(StretchVer && StretchHor)
								screen[XCoord+w+1][YCoord+h+1]	= fgColor;
						}
					//if BitToTest is greater 8 we increase the DotAddress to fetch the next byte
					//and set the BitToTest to 0
					if(BitToTest == 8)
					{
						DotAddress++;
						BitToTest	= 0;
					}
				}
			}
		}
		//now the multicolor variante
		if(MultiColor && DrawSprite || UserMcmMode)
		{
			//we compute the needed hor. and vert. lines depending on stretching
			byte rows			= StretchVer?42:21;
			byte cols			= StretchHor?48:24;
			//we get the foreground colors, in mcm-mode we have 3 registers
			byte fgColor10		= peek(0xD027 + s);
			byte fgColor01		= peek(0xD025);
			byte fgColor11		= peek(0xD026);
			byte fgColor; //we will compute this later
			//this will be the bit to test
			byte BitToTest		= 0;
			//a little array for reversing bits later
			byte bits[]		= {7,6,5,4,3,2,1,0};
			//a variable to check later if we are in the first or second bit of the bitpair
			bool second_run	= false;
			//now we loop through the sprite dot area
			for(int h=0; h<rows; h+=(rows/21))
			{
				for(int w=0; w<cols; w+=(cols/24))
				{
					if(!second_run)
					{
                        //we test the bit pairs
						if(	TestBit(peek(DotAddress), bits[BitToTest])
							&& !TestBit(peek(DotAddress), bits[BitToTest+1]))
						{
							fgColor		= fgColor10;
						}
						else if(	!TestBit(peek(DotAddress), bits[BitToTest])
									&& !TestBit(peek(DotAddress), bits[BitToTest+1]))
						{
							fgColor		= 0;
						}
						else if(	!TestBit(peek(DotAddress), bits[BitToTest])
									&& TestBit(peek(DotAddress), bits[BitToTest+1]))
						{
							fgColor		= fgColor01;
						}
						else if(	TestBit(peek(DotAddress), bits[BitToTest])
									&& TestBit(peek(DotAddress), bits[BitToTest+1]))
						{
							fgColor		= fgColor11;
						}
						BitToTest	+= 2;
					}
					if( fgColor != 0	&&
						(XCoord) >= 0	&&
						(XCoord+w+1) <= 320	&&
						(YCoord) >= 0	&&
						(YCoord+h+1) <= 200)
					{
						screen[XCoord+w][YCoord+h]			= fgColor;
						if(StretchHor)
							screen[XCoord+w+1][YCoord+h]	= fgColor;
						if(StretchVer)
							screen[XCoord+w][YCoord+h+1]	= fgColor;
						if(StretchVer && StretchHor)
							screen[XCoord+w+1][YCoord+h+1]	= fgColor;

					}
					//if BitToTest is greater 8 we increase the DotAddress to fetch the next byte
					//and set the BitToTest to 0
					if(BitToTest > 7)
					{
						DotAddress++;
						BitToTest	= 0;
					}
					if(!second_run)
						second_run = true;
					else
						second_run = false;
				}
			}
		}
	}
}


//////////////////////////////////////////////////////////////////
// Bitmap Mode functions
//////////////////////////////////////////////////////////////////
void vic_drawBitmapModeScreen()
{
		//first we fetch the current positions of the color memory and the
		//bitmap dot memory relative to the current VIC-II Bank
        unsigned short color_mem_pos	= VicBank + VicScreenMemory;
		//bitmap fot area is 8k, so only 2 areas can be theoretical in a vic bank
		//thats why only one bit(bit 3) is needed to check if the bitmap area is beginning in 0x0000 or 0x2000
        unsigned short bitmap_dotmatrix = VicBank + (TestBit(peek(0xD018),3)?0x2000:0);
        unsigned short bmp_count		= 0;
		//X and Y coord of the 8x8 block for later usage
		short XCoord					= 0;
		short YCoord					= 0;
		//8 bytes form a 8x8 pixel block of bitmap info
		//the entire screen uses (320*200)/8 bytes (= 8000 bytes)
        for( int bmp_data=bitmap_dotmatrix; bmp_data<(bitmap_dotmatrix+8000); bmp_data+=0x8 )
        {
			//first we check if we have to draw the bitmap block in multicolor

			//no multicolor needed
			if(!MultiColorMode && !UserMcmMode)
			{
				//no multicolor, so the 2 colors we use are
				//color 1: bit 0..3 of the color mem byte
				//color 2: bit 4..7 of the color mem byte
				byte color1				= peek(color_mem_pos+bmp_count) & 0xF;
				byte color2				= peek(color_mem_pos+bmp_count) >> 4;
				byte realcolor;
				//a little array for reversing bits later
				byte bits[]				= {7,6,5,4,3,2,1,0};
				//now we draw the bitmap block
				for(int h=0; h<8; h++)
				{
					//we fetch the current byte with the bitmap data
					byte BmpData		= peek(bmp_data+h);
					//we loop through the bits of the byte and set the screen pixel
					for(int w=0; w<8; w++)
					{
						if(TestBit(BmpData, bits[w]))
							realcolor	= color2;
						else
							realcolor	= color1;

						screen[XCoord+w][YCoord+h]	= realcolor;
					}
				}
				//lets compute the XCoord and YCoord of the next 8x8 block
				XCoord+=8;
				if(XCoord >= 319)
				{
					XCoord	= 0;
					YCoord	+= 8;
				}
			}

			//and now the multicolor variante
			else if(MultiColorMode || UserMcmMode)
			{
				//multicolor, so there are  4 colors we use 
				//color 00: 0xD021
				//color 01: bits 4..7 of the color mem byte
				//color 10: bits 0..3 of the color mem byte
				//color 11: byte from character color memory at 0xD800...
				byte color00			= peek(0xD021);
				byte color01			= peek(color_mem_pos+bmp_count) >> 4;
				byte color10			= peek(color_mem_pos+bmp_count) & 0xF;
				byte color11			= peek(0xD800+bmp_count);
				byte realcolor;
				//a little array for reversing bits later
				byte bits[]				= {7,6,5,4,3,2,1,0};
				//now we draw the bitmap block
				for(int h=0; h<8; h++)
				{
					//we fetch the current byte with the bitmap data
					byte BmpData		= peek(bmp_data+h);				
					//we loop through the bits of the byte and set the screen pixel
					for( int w=0; w<8; w+=2) // the block width is 4 pixel stretched to 8
					{
						//here we check the bit of the "line-byte" 
						//( 8 bits of course ) for its value
						//and setting the color (we have to reverse the bits here)
						byte ColorValue1	= TestBit(BmpData, bits[w]);
						byte ColorValue2	= TestBit(BmpData, bits[w+1]);
						byte ColorValue		= (ColorValue1 << 1) | ColorValue2;
						if(ColorValue == 0)		 //00
							realcolor		= color00; 
						else if(ColorValue == 1) //01
							realcolor		= color01; 
						else if(ColorValue == 2) //10
							realcolor		= color10; 
						else if(ColorValue == 3) //11
							realcolor		= color11; 
						//at last we set the fetched color in our Screen Matrix
						screen[XCoord + w][YCoord + h]		= realcolor;
						screen[XCoord + w + 1][YCoord + h]	= realcolor;
					}
				}
				//lets compute the XCoord and YCoord of the next 8x8 block
				XCoord+=8;
				if(XCoord >= 319)
				{
					XCoord	= 0;
					YCoord	+= 8;
				}
			}
            //lets fetch the next 8x8 pixel block in our loop
			bmp_count++;
        }
}
//////////////////////////////////////////////////////////////////
// Vic Update
//////////////////////////////////////////////////////////////////
void VicUpdate()
{
	vic_peek = true;

	//
	//get vic bank from 0xDD00
	//

	//check if a user defined VIC bank exists and use it 
	if(VicUserBank != 1)
		VicBank = VicUserBank;
	else
	{
		bool bit0 = TestBit(peek(0xDD00),0);
		bool bit1 = TestBit(peek(0xDD00),1);	
		if(!bit0 && !bit1) VicBank = 0xC000; //$C000-$FFFF
		if(!bit0 &&  bit1) VicBank = 0x8000; //$8000-$BFFF
		if(bit0 &&  !bit1) VicBank = 0x4000; //$4000-$7FFF
		if(bit0 &&   bit1) VicBank = 0x0000; //$0000-$3FFF
	}

	//
	//get vic screen memory from bite 5..8 from 0xD018
	//

	//check if a user defined VIC screen memory exists and use it 
	if(VicUserScreenMemory != 1)
		VicScreenMemory = VicUserScreenMemory;
	else
	{
		VicScreenMemory	= (peek(0xD018) >> 4) * 0x400;
	}

	//
	// get the charset start address
	//

	/*
	- bits 1 .. 4 from address 0xD018 defines the start address of the character set
	- address is computed: bit 1 .. 4 * 0x800
	*/

	//check if a user defined VIC char set start address exists and use it 
	if(VicUserAddress != 1)
		VicAddress = VicUserAddress;
	else
	{
		byte nybble;

		//fetch the bits 1 to 4 of address =0xD018
		nybble			= (peek(0xD018) & 0xF) >> 1;

		//compute the char set start address 
		VicAddress		= nybble * 0x800;
	}

	if(TestBit(peek(0xD016),4))		
	{
		MultiColorMode	= true;
		if(infoDlg.cEdit18) 
			infoDlg.cEdit18.SetWindowText("yes");

		if(!TestBit(peek(0xD011),5)) debug("Multicolor-Mode activated!");
	}
	else
	{
		if(infoDlg.cEdit18) 
			infoDlg.cEdit18.SetWindowText("no");
		MultiColorMode	= false;
	}
	if(!UserTextMode && !UserBmpMode)
	{
		if(!TestBit(peek(0xD011),5))
		{
			if(infoDlg.cEdit16) 
				infoDlg.cEdit16.SetWindowText("yes");
			if(infoDlg.cEdit17) 
				infoDlg.cEdit17.SetWindowText("no");
			vic_drawTextModeScreen();  //Bit 5 from 0xD011 is 0: TextMode
		}
		else if(TestBit(peek(0xD011),5))
		{
			if(infoDlg.cEdit16) 
				infoDlg.cEdit16.SetWindowText("no");
			if(infoDlg.cEdit17) 
				infoDlg.cEdit17.SetWindowText("yes");
			vic_drawBitmapModeScreen();
		}
	}
	else
	{
		if(UserTextMode)
		{
			if(infoDlg.cEdit16) 
				infoDlg.cEdit16.SetWindowText("yes");
			if(infoDlg.cEdit17) 
				infoDlg.cEdit17.SetWindowText("no");
			vic_drawTextModeScreen();  
		}
		else if(UserBmpMode)
		{
			if(infoDlg.cEdit16) 
				infoDlg.cEdit16.SetWindowText("no");
			if(infoDlg.cEdit17) 
				infoDlg.cEdit17.SetWindowText("yes");
			vic_drawBitmapModeScreen();
		}
	}
	vic_drawSpriteScreen();

	//Output some debugging info
	DebugStr.Format("0x%x",VicBank);
	infoDlg.cEdit19.SetWindowText(DebugStr);
	DebugStr.Format("0x%x",VicBank + VicAddress);
	infoDlg.cEdit20.SetWindowText(DebugStr);
	DebugStr.Format("0x%x",VicBank + VicScreenMemory);
	infoDlg.cEdit21.SetWindowText(DebugStr);
}

//////////////////////////////////////////////////////////////////
// main draw function
//////////////////////////////////////////////////////////////////
void display()
{
        if(!go)
        {
                go = true;
				vic_peek = false;
				
                cpu_run();				
        }
		vic_peek = true;
        // switch to projection mode
        glMatrixMode(GL_PROJECTION);
        // save previous matrix which contains the
        //settings for the perspective projection
        glPushMatrix();
        // reset matrix
        glLoadIdentity();
        // set a 2D orthographic projection
        gluOrtho2D(0, 320, 0, 200);
        // invert the y axis, down is positive
        glScalef(1, -1, 1);
        // mover the origin from the bottom left corner
        // to the upper left corner
        glTranslatef(0, -200, 0);
        glMatrixMode(GL_MODELVIEW);
        glPushMatrix();
        glLoadIdentity();

		
		//VicUpdate();
        //////////////////////////////////////////////////////////////////
        // draw screen from the screen-array
        //////////////////////////////////////////////////////////////////
		for(int x=0;x<320;x++)
        {


                for(int y=0;y<200;y++)
                {
                        cl=screen[x][y];

                        glColor3ub(colors[cl][0],colors[cl][1],colors[cl][2]);
                        glBegin(GL_POLYGON);
                                glVertex2f(x,y);
                                glVertex2f(x+1.0,y);
                                glVertex2f(x+1.0,y+1.0);
                                glVertex2f(x,y+1.0);
                        glEnd();

                }
        }
        glPopMatrix();
        glutPostRedisplay();
		vic_peek = false;

}
//////////////////////////////////////////////////////////////////
// keyscan
//////////////////////////////////////////////////////////////////
void checkKey(unsigned char key, int x, int y)
{
        keyDown = true;
        if(glutGetModifiers() == GLUT_ACTIVE_SHIFT)
        {
                byte row_bit = 6;
                byte col_bit = 4;
                keymatrix_row[row_bit] = ClearBit(keymatrix_row[row_bit], col_bit);
                for(int i=0; i<sizeof(GlutKey); i++)
                {
                        if(GlutShiftedKey[i] == key)
                        {
                                byte row_bit = RowToSet[i];
                                byte col_bit = ColToSet[i];
                                keymatrix_row[row_bit] = ClearBit(keymatrix_row[row_bit], col_bit);
                        }
                }
        }
        else
        {
                byte row_bit = 6;
                byte col_bit = 4;
                keymatrix_row[row_bit] = SetBit(keymatrix_row[row_bit], col_bit);
                for(int i=0; i<sizeof(GlutKey); i++)
                {
                        if(GlutKey[i] == key)
                        {
								if(key == 0xD)
								{
									//Return = Joystick 1 and 2 Fire
									//joy1 = ClearBit(joy1,0x04);
								}
                                byte row_bit = RowToSet[i];
                                byte col_bit = ColToSet[i];
                                keymatrix_row[row_bit] = ClearBit(keymatrix_row[row_bit], col_bit);
                        }
                }
        }
        if(key == 27) exit(true); //ESC pressed
}
void checkUpKey(unsigned char key, int x, int y)
{
        if(glutGetModifiers() == GLUT_ACTIVE_SHIFT)
        {
                byte row_bit = 6;
                byte col_bit = 4;
                keymatrix_row[row_bit] = ClearBit(keymatrix_row[row_bit], col_bit);
                for(int i=0; i<sizeof(GlutKey); i++)
                {
                        if(GlutShiftedKey[i] == key)
                        {
                                byte row_bit = RowToSet[i];
                                byte col_bit = ColToSet[i];
                                keymatrix_row[row_bit] = SetBit(keymatrix_row[row_bit], col_bit);
                        }
                }
        }
        else
        {
                byte row_bit = 6;
                byte col_bit = 4;
                keymatrix_row[row_bit] = SetBit(keymatrix_row[row_bit], col_bit);
                for(int i=0; i<sizeof(GlutKey); i++)
                {
                        if(GlutKey[i] == key)
                        {
								if(key == 0xD)
								{
									//Return = Joystick 1 and 2 Fire
									//joy1 = SetBit(joy1,0x04);
								}
                                byte row_bit = RowToSet[i];
                                byte col_bit = ColToSet[i];
                                keymatrix_row[row_bit] = SetBit(keymatrix_row[row_bit], col_bit);
                        }
                }
        }
}
void checkSpecial(int x, int y, int z)
{

        if(x == 0xA) debugMode = true;
        byte key = (byte)x;
        for(int i=0; i<sizeof(GlutKey); i++)
        {
                if(GlutKey[i] == key+80)
                {
                        if(	key+80 ==  0xB4 || 
							key+80 ==  0xB6 || 
							key+80 ==  0xB7 || 
							key+80 ==  0xB5 || 
							key+80 ==  0xBC)
                        {
                                //Joystick1 = Joystick 2 at the moment
                                joy1 = ClearBit(joy1,ColToSet[i]);
                        }
                        else
                        {
                                byte row_bit = RowToSet[i];
                                byte col_bit = ColToSet[i];
                                keymatrix_row[row_bit] = ClearBit(keymatrix_row[row_bit], col_bit);
                        }
                }
        }

}
void checkUpSpecial(int x, int y, int z)
{
        if(x == 0xA) debugMode = true;
        byte key = (byte)x;
		MemoryValue.Format("%X",(key+80));
		debug(MemoryValue+" gedrckt! <Special>");
        for(int i=0; i<sizeof(GlutKey); i++)
        {
                if(GlutKey[i] == key+80)
                {
                        if( key+80 ==  0xB4 || 
							key+80 ==  0xB6 || 
							key+80 ==  0xB7 || 
							key+80 ==  0xB5 || 
							key+80 ==  0xBC)
                        {
                                joy1 = SetBit(joy1,ColToSet[i]);
                        }
                        else
                        {
                                byte row_bit = RowToSet[i];
                                byte col_bit = ColToSet[i];
                                keymatrix_row[row_bit] = SetBit(keymatrix_row[row_bit], col_bit);
                        }
                }
        }
}
void vic_run(int argc, char **argv)
{
	
        glutInit(&argc, argv);
        glutInitDisplayMode(GLUT_SINGLE|GLUT_RGB);
        glutGameModeString("1280x1024:16@85") ;
        //glutEnterGameMode()   ;
        glutInitWindowSize(640, 400);
        glutInitWindowPosition(100,100);
        glutCreateWindow("Emulator");
		glutDisplayFunc(display);
        glutKeyboardFunc(checkKey);
        glutKeyboardUpFunc(checkUpKey);
        glutSpecialFunc(checkSpecial);
        glutSpecialUpFunc(checkUpSpecial);
		debug("VIC Emulation startet");
        glutMainLoop();
		
}






